;
;
; VECVOICE.I
;
;
; Copyright (c) 2002 Alex Herbert
;
;



;
; Include the serial driver
;

        include "SER_JI.I"


;
; Allophone Constants
;

VV_PA1          equ     $00     ; 20ms pause (before BB,DD,GG and JH)
VV_PA2          equ     $01     ; 40ms pause (before BB,DD,GG and JH)
VV_PA3          equ     $02     ; 60ms (before PP,TT,KK,CH and between words)
VV_PA4          equ     $03     ; 100ms pause (between clauses and sentences)
VV_PA5          equ     $04     ; 200ms (between clauses and sentences)
VV_OY           equ     $05     ; nOIse, tOY, vOIce 
VV_AY           equ     $06     ; kIte, skY, mIGHty
VV_EH           equ     $07     ; ExtEnt, gEntlemEn
VV_KK3          equ     $08     ; Crane, Quick, Clown, sCream
VV_PP           equ     $09     ; Pleasure, amPle, triP
VV_JH           equ     $0a     ; JudGe, inJure
VV_NN1          equ     $0b     ; earN, Night
VV_IH           equ     $0c     ; sIttIng, strandEd
VV_TT2          equ     $0d     ; TesT, sTreeT
VV_RR1          equ     $0e     ; Read, WRite, x-Ray
VV_AX           equ     $0f

VV_MM           equ     $10     ; Milk, alarM, aMple
VV_TT1          equ     $11     ; tesTs, its, (before SS)
VV_DH1          equ     $12     ; THis, THen, THey
VV_IY           equ     $13     ; trEAt, pEOple, pennY
VV_EY           equ     $14     ; geEAt, stAte, trAY
VV_DD1          equ     $15     ; playeD, enD
VV_UW1          equ     $16     ; yOU, compUter, (after YY)
VV_AO           equ     $17     ; tAlking, sOng
VV_AA           equ     $18     ; pOttery, cOtton
VV_YY2          equ     $19     ; Yes, Yarn, Yo-Yo
VV_AE           equ     $1a     ; extrAct, Acting
VV_HH1          equ     $1b     ; (before YR,IY,IH,EY,EH,XR,AE)
VV_BB1          equ     $1c     ; riB, fiBBer, Bleed, Brown
VV_TH           equ     $1d     ; THought
VV_UH           equ     $1e     ; cOOkie, fUll
VV_UW2          equ     $1f     ; tOO, fOOd

VV_AW           equ     $20     ; sOUnd, mOUse, dOWn
VV_DD2          equ     $21     ; Down, Drain
VV_GG3          equ     $22     ; anGer, peG
VV_VV           equ     $23     ; Vest, proVe, eVen
VV_GG1          equ     $24     ; Gear, Gift
VV_SH           equ     $25     ; SHirt, leaSH, naTion
VV_ZH           equ     $26     ; beiGe, pleaSure
VV_RR2          equ     $27     ; bRown, cRane, gRease
VV_FF           equ     $28     ; Fisrt, driFt
VV_KK2          equ     $29     ; speaK, tasK
VV_KK1          equ     $2a     ; Cute, Clown, sCream
VV_ZZ           equ     $2b     ; Zoo, phaSe
VV_NG           equ     $2c     ; striNG, aNger
VV_LL           equ     $2d     ; Like, heLLo, steeL
VV_WW           equ     $2e     ; We, Warrant, lingUist
VV_XR           equ     $2f     ; hAIR, declARE, stARE

VV_WH           equ     $30     ; WHite, WHim, tWenty
VV_YY1          equ     $31     ; (in middle of words)
VV_CH           equ     $32     ; ChurCH, feaTure
VV_ER1          equ     $33     ; lettER, intERRupt
VV_ER2          equ     $34     ; bIRd, fERn, bURn
VV_OW           equ     $35     ; zOne, clOse, snOW
VV_DH2          equ     $36     ; baTHE, baTHing 
VV_SS           equ     $37     ; Street, bliSter
VV_NN2          equ     $38     ; (before UH,OW,OY,OR,AR,AA)
VV_HH2          equ     $39     ; (before UH,OW,OY,AO,OR,AR)
VV_OR           equ     $3a     ; fORtune, adORn, stORE
VV_AR           equ     $3b     ; fARm, alARm
VV_YR           equ     $3c     ; hEAR, EARring, IRresponsible
VV_GG2          equ     $3d     ; Green, Glue
VV_EL           equ     $3e     ; littLE, angLE, gentLEmen
VV_BB2          equ     $3f     ; Beast, Bait

VV_TERM         equ     $ff     ; Allophone string terminator



;
; Variables
;
; These routines require 2 bytes of RAM for the following variable:
;
; vv_addr - 16-bit pointer to next allophone.
;
; You may wish to comment out the following lines and declare vv_addr
; elsewhere in your code.
;


        bss

vv_addr         ds      2



;
; Subroutines
;


        code


;
; vv_init
; -------
;
; Function:
;       Initialize allophone pointer.
;
; Usage:
;       Call vv_init once at the start of the code.
;


vv_init
        ldx     #vv_silence
        stx     vv_addr         ; point to 'silence' allophone string
        rts



;
; vv_speak
; --------
;
; Function:
;       Sends allophones to the VecVoice.
;
; Usage:
;       Call vv_speak once per frame. (I.e. stick it in your main loop)
;       See note below.
;
;       Then, just store the start address of an allophone string in
;       vv_addr when you want the VecVoice to speak, and let this
;       function do the work!
;
;       To abort speech in progress, point vv_addr at vv_silence or
;       call vv_init.
;
; Note:
;    1. This subroutine requires that the current joystick button
;       status is held at RAM address $c80f (TRIGGR).  This is usually
;       set by calling the Executive ROM routine INPUT (a.k.a.
;       read_switches2) at $F1BA.  For best speech quality, it's
;       reccommended that vv_speak is called immediately after
;       reading the joystick buttons.
;
;    2. Allophone strings should be terminated with VV_TERM.
;
; Caution:
;       Contents of the dp register may (or may not) be set to $d0.
;


vv_speak
        lda     $c80f           ; get joystick buttons (TRIGGR)
        bita    #$f0            ; READY (LRQ) signal detected?
        beq     vv_exit

        ldx     vv_addr         ; get allophone pointer
        lda     ,x+             ; read next allophone
        bmi     vv_exit         ; exit if end of string
        stx     vv_addr         ; store allophone pointer

        ldb     #$d0
        tfr     b,dp            ; direct page = $d0

        jmp     ser_txbyte      ; send byte

vv_exit
        rts



;
; Silence string
;
; Initial speech string.  Point vv_addr to here to abort speech
; in progress.
;


vv_silence
        db      VV_PA1,VV_TERM



